function [du,dc] = l2solve(u,c,rhs,u0,M,maxits,alpha,beta)
%L"SOLVE    Solve quadratic L2-constrained IRGN-subproblem.
%   [DU,DC] = L2SOLVE(U,C,RHS,U0,M,MAXITS,ALPHA,BETA) computes the updates
%   DU, DC for image and coil sensitivities in the IRGN method for given
%   iterates U,C, gradient RHS, initial guess U0, linear operator M with
%   MAXITS iterations and L2 penalty ALPHA for the sensitivities and L2
%   penalty BETA for the image.

% July 2011,
% Christian Clason (christian.clason@uni-graz.at)

[n,m,nc] = size(c);

% estimate operator norm using power iteration
x1  = rand(n,m); x2 = rand(n,m,nc);
[y1,y2] = M(x1,x2);
for i=1:10
    if norm(y1(:))~=0
        x1 = y1./norm(y1(:));
    else
        x1 = y1;
    end
    x2 = y2./norm(y2(:));
    [y1,y2] = M(x1,x2);
    l1 = y1(:)'*x1(:);
    l2 = y2(:)'*x2(:);
end
L = 2*max(abs(l1),abs(l2)); % Lipschitz constant estimate

tau   = 1/sqrt(8+(L));      % primal step size

rhsu = rhs(:,:,1);
rhsc = rhs(:,:,2:end);

% initialize iterates du, dc
du = zeros(n,m);
dc = zeros(n,m,nc);

% initialize leading points
ub = zeros(n,m);
cb = zeros(n,m,nc);

for iter = 1:maxits
    
    % update primal (du,dc): F'*F du  + F(u) + (u+ub-u0)
    [Mu,Mc] = M(ub,cb);
    eta3 = Mu + rhsu + beta*(u+ub-u0);      % primal variable: image
    eta4 = Mc + rhsc + alpha*(c+cb);        % primal variable: coils
    uold = du;
    cold = dc;
    du   = du - tau * eta3;
    dc   = dc - tau * eta4;
    
    % update primal leading points
    ub   = 2*du - uold;
    cb   = 2*dc - cold;
    
end
