function [du,dc] = tvsolve(u,c,rhs,u0,M,maxits,alpha,beta)
%TVSOLVE    Solve quadratic tv-constrained IRGN-subproblem.
%   [DU,DC] = TVSOLVE(U,C,RHS,U0,M,MAXITS,ALPHA,BETA) computes the updates
%   DU, DC for image and coil sensitivities in the IRGN method for given
%   iterates U,C, gradient RHS, initial guess U0, linear operator M with
%   MAXITS iterations and L2 penalty ALPHA for the sensitivities and TV
%   penalty BETA for the image.

% July 2011,
% Christian Clason (christian.clason@uni-graz.at)

[n,m,nc] = size(c);

proj = @(x1,x2) max(1,sqrt(x1.^2+x2.^2)/beta);  % projection on C_beta
dx   = @(x) [diff(x,1,2), x(:,1)-x(:,m)];       % discrete x-derivative
dy   = @(x) [diff(x,1,1); x(1,:)-x(n,:)];       % discrete y-derivative
dxt  = @(x) [x(:,m)-x(:,1), -diff(x,1,2)];      % transpose x-derivative
dyt  = @(x) [x(n,:)-x(1,:); -diff(x,1,1)];      % transpose y-derivative

% estimate operator norm using power iteration
x1  = rand(n,m); x2 = rand(n,m,nc);
[y1,y2] = M(x1,x2);
for i=1:10
    if norm(y1(:))~=0
        x1 = y1./norm(y1(:));
    else
        x1 = y1;
    end
    x2 = y2./norm(y2(:));
    [y1,y2] = M(x1,x2);
    l1 = y1(:)'*x1(:);
    l2 = y2(:)'*x2(:);
end
L = 2*max(abs(l1),abs(l2)); % Lipschitz constant estimate

sigma = 1/sqrt(8+(L));      % dual step size
tau   = 1/sqrt(8+(L));      % primal step size

rhsu = rhs(:,:,1);
rhsc = rhs(:,:,2:end);

% initialize iterates p=(p1,p2), du, dc
p1 = zeros(n,m);
p2 = zeros(n,m);

du = zeros(n,m);
dc = zeros(n,m,nc);

% initialize leading points \bar du, \bar dc
ub  = du; cb = dc;

for iter = 1:maxits
    
    % update dual (p): (eta1,eta2) = nabla(u+du-u0)
    ukp = ub + u - u0;
    eta1 = dx(ukp);
    eta2 = dy(ukp);
    y1 = p1 + sigma * eta1;
    y2 = p2 + sigma * eta2;
    my = proj(y1,y2);
    p1 = y1./my;
    p2 = y2./my;
    
    % update primal (du,dc): F'*F du  + F(u) - div(p)
    [Mu,Mc] = M(ub,cb);
    eta3 = Mu + rhsu + dxt(p1)+dyt(p2); % primal variable: image
    eta4 = Mc + rhsc + alpha*(c+cb);    % primal variable: coils
    uold = du;
    cold = dc;
    du   = du - tau * eta3;
    dc   = dc - tau * eta4;
    
    % update primal leading points
    ub   = 2*du - uold;
    cb   = 2*dc - cold;
     
end

